/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.fhirendpoint;

import ca.cdr.api.util.TransactionLogRequestDetailsUtil;
import ca.uhn.fhir.interceptor.api.Hook;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.fhir.interceptor.api.Pointcut;
import ca.uhn.fhir.rest.api.RestOperationTypeEnum;
import ca.uhn.fhir.rest.server.servlet.ServletRequestDetails;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * This interceptor provides an example of how to add key/value pairs to the request.  The pairs will subsequently be serialized into an
 * inline Json String and added the transaction log.
 */
@SuppressWarnings({"unused"})
@Interceptor
public class TransactionLogAdditionalJsonInterceptor {

	private static final Logger ourLog = LoggerFactory.getLogger(TransactionLogAdditionalJsonInterceptor.class);

	@Hook(Pointcut.SERVER_INCOMING_REQUEST_PRE_HANDLED)
	public void incomingRequestPreHandled(RestOperationTypeEnum theOperation, ServletRequestDetails theRequestDetails) {
		List<String> extraValues = List.of("Tuberculosis", "Polio", "Malaria", "Dengue fever");

		ourLog.info("TransactionLogAdditionalJsonInterceptor SERVER_INCOMING_REQUEST_PRE_HANDLED - started");

		// the values provided needs to be serializable with Jackson 2.0.
		// since they will be rendered as inline json in the transaction log entry
		TransactionLogRequestDetailsUtil.addAdditionalJsonProperty(theRequestDetails, "clinic", "Acme Ouest");
		TransactionLogRequestDetailsUtil.addAdditionalJsonProperty(theRequestDetails, "diseases", extraValues);

		ourLog.info("TransactionLogAdditionalJsonInterceptor SERVER_INCOMING_REQUEST_PRE_HANDLED - ended");
	}
}
